from __future__ import annotations
from datetime import datetime, date
from enum import Enum

from typing import List, Dict, Optional, Any, Union
from pydantic import BaseModel as BaseModel, ConfigDict,  Field, field_validator
import re
import sys
if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal


metamodel_version = "None"
version = "None"

class ConfiguredBaseModel(BaseModel):
    model_config = ConfigDict(
        validate_assignment=True,
        validate_default=True,
        extra = 'forbid',
        arbitrary_types_allowed=True,
        use_enum_values = True)
    pass


class CreatorOrMaintainerEnum(str, Enum):
    """
    The entity responsible for maintaining a dataset.
    """
    
    Person = "Person"
    
    Organization = "Organization"
    
    

class MediaTypeEnum(str, Enum):
    
    
    csv = "csv"
    
    rdf_xml = "rdf-xml"
    
    

class FormatEnum(str, Enum):
    
    
    JSON_LD = "JSON-LD"
    
    N3 = "N3"
    
    N_Triples = "N-Triples"
    
    N_Quads = "N-Quads"
    
    LD_Patch = "LD Patch"
    
    Microdata = "Microdata"
    
    OWL_XML_Serialization = "OWL XML Serialization"
    
    OWL_Functional_Syntax = "OWL Functional Syntax"
    
    OWL_Manchester_Syntax = "OWL Manchester Syntax"
    
    POWDER = "POWDER"
    
    POWDER_S = "POWDER-S"
    
    PROV_N = "PROV-N"
    
    PROV_XML = "PROV-XML"
    
    RDFa = "RDFa"
    
    RDFSOLIDUSJSON = "RDF/JSON"
    
    RDFSOLIDUSXML = "RDF/XML"
    
    RIF_XML_Syntax = "RIF XML Syntax"
    
    SPARQL_Results_in_XML = "SPARQL Results in XML"
    
    SPARQL_Results_in_JSON = "SPARQL Results in JSON"
    
    SPARQL_Results_in_CSV = "SPARQL Results in CSV"
    
    SPARQL_Results_in_TSV = "SPARQL Results in TSV"
    
    Turtle = "Turtle"
    
    TriG = "TriG"
    
    YAML = "YAML"
    
    JSON = "JSON"
    
    

class CompressionEnum(str, Enum):
    
    
    GZIP = "GZIP"
    
    TAR = "TAR"
    
    TARGZIP = "TARGZIP"
    
    ZIP = "ZIP"
    
    

class EncodingEnum(str, Enum):
    
    
    ASCII = "ASCII"
    
    Big5 = "Big5"
    
    EUC_JP = "EUC-JP"
    
    EUC_KR = "EUC-KR"
    
    EUC_TW = "EUC-TW"
    
    GB2312 = "GB2312"
    
    HZ_GB_2312 = "HZ-GB-2312"
    
    ISO_2022_CN_EXT = "ISO-2022-CN-EXT"
    
    ISO_2022_CN = "ISO-2022-CN"
    
    ISO_2022_JP_2 = "ISO-2022-JP-2"
    
    ISO_2022_JP = "ISO-2022-JP"
    
    ISO_2022_KR = "ISO-2022-KR"
    
    ISO_8859_10 = "ISO-8859-10"
    
    ISO_8859_11 = "ISO-8859-11"
    
    ISO_8859_13 = "ISO-8859-13"
    
    ISO_8859_14 = "ISO-8859-14"
    
    ISO_8859_15 = "ISO-8859-15"
    
    ISO_8859_16 = "ISO-8859-16"
    
    ISO_8859_1 = "ISO-8859-1"
    
    ISO_8859_2 = "ISO-8859-2"
    
    ISO_8859_3 = "ISO-8859-3"
    
    ISO_8859_4 = "ISO-8859-4"
    
    ISO_8859_5 = "ISO-8859-5"
    
    ISO_8859_6 = "ISO-8859-6"
    
    ISO_8859_7 = "ISO-8859-7"
    
    ISO_8859_8 = "ISO-8859-8"
    
    ISO_8859_9 = "ISO-8859-9"
    
    KOI8_R = "KOI8-R"
    
    KOI8_U = "KOI8-U"
    
    Shift_JIS = "Shift_JIS"
    
    UTF_16 = "UTF-16"
    
    UTF_32 = "UTF-32"
    
    UTF_7 = "UTF-7"
    
    UTF_8 = "UTF-8"
    
    

class NamedThing(ConfiguredBaseModel):
    """
    A generic grouping for any identifiable entity
    """
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class Information(ConfiguredBaseModel):
    """
    Grouping for datasets and data files
    """
    compression: Optional[CompressionEnum] = Field(None, description="""The compression format of the data. This is not the same as the media type. Rather, this is the compression format of the data in a more specific sense, e.g., zip, gzip, etc.""")
    conforms_to: Optional[str] = Field(None, description="""The standard to which the data conforms. This is not the same as the media type. Rather, this is the standard to which the data conforms in a more specific sense, e.g., frictionless, schema.org, etc.""")
    conforms_to_class: Optional[str] = Field(None, description="""The class in the schema to which the data object instantiates.""")
    conforms_to_schema: Optional[str] = Field(None, description="""The schema to which the data conforms. This is not the same as the media type. Rather, this is the schema to which the data conforms in a more specific sense, and even more specific than the general set of standards it conforms to.""")
    created_by: Optional[List[str]] = Field(default_factory=list, description="""Agent that created the element""")
    created_on: Optional[str] = Field(None, description="""Date and Time at which the element was created""")
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    doi: Optional[str] = Field(None, description="""The Digital Object Identifier of the data, with the doi prefix.""")
    download_url: Optional[str] = Field(None, description="""URL from which the data can be downloaded. This is not the same as the landing page, which is a page that describes the dataset. Rather, this URL points directly to the data itself.""")
    id: str = Field(..., description="""the unique name of the dataset""")
    issued: Optional[str] = Field(None)
    keywords: Optional[List[str]] = Field(default_factory=list, description="""Keywords associated with the data. These may be provided by the data creator or assigned later in a manual or automated manner.""")
    language: Optional[str] = Field(None, description="""language in which the information is expressed""")
    last_updated_on: Optional[str] = Field(None, description="""Date and Time at which the element was last updated""")
    license: Optional[str] = Field(None, description="""license for the data""")
    modified_by: Optional[str] = Field(None, description="""agent that modified the element""")
    page: Optional[str] = Field(None)
    publisher: Optional[str] = Field(None)
    status: Optional[str] = Field(None, description="""Status of the element in terms of its maturity or life cycle""")
    title: Optional[str] = Field(None, description="""the official title of the element""")
    version: Optional[str] = Field(None, description="""particular version of schema""")
    was_derived_from: Optional[str] = Field(None, description="""A derivation is a transformation of an entity into another, an update of an entity resulting in a new one, or the construction of a new entity based on a pre-existing entity.@en""")
    
    

class FormatDialect(ConfiguredBaseModel):
    """
    Additional format information for a file
    """
    comment_prefix: Optional[str] = Field(None)
    delimiter: Optional[str] = Field(None)
    double_quote: Optional[str] = Field(None)
    header: Optional[str] = Field(None)
    quote_char: Optional[str] = Field(None)
    
    

class Person(NamedThing):
    """
    An individual human being.
    """
    affiliation: Optional[List[str]] = Field(default_factory=list, description="""The organization(s) to which the person belongs.""")
    email: Optional[str] = Field(None, description="""The email address of the person.""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class Organization(NamedThing):
    """
    A collection of people acting in common interests.
    """
    email: Optional[str] = Field(None, description="""The email address of the organization.""")
    ror_id: Optional[str] = Field(None, description="""Unique ROR identifier.""")
    wikidata_id: Optional[str] = Field(None, description="""Unique Wikidata identifier.""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class DatasetProperty(NamedThing):
    """
    Represents a single property of a dataset, or a set of related properties.
    """
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class DatasetCollection(Information):
    """
    A collection of related datasets, likely containing multiple files of multiple potential purposes and properties.
    """
    resources: Optional[List[str]] = Field(default_factory=list)
    compression: Optional[CompressionEnum] = Field(None, description="""The compression format of the data. This is not the same as the media type. Rather, this is the compression format of the data in a more specific sense, e.g., zip, gzip, etc.""")
    conforms_to: Optional[str] = Field(None, description="""The standard to which the data conforms. This is not the same as the media type. Rather, this is the standard to which the data conforms in a more specific sense, e.g., frictionless, schema.org, etc.""")
    conforms_to_class: Optional[str] = Field(None, description="""The class in the schema to which the data object instantiates.""")
    conforms_to_schema: Optional[str] = Field(None, description="""The schema to which the data conforms. This is not the same as the media type. Rather, this is the schema to which the data conforms in a more specific sense, and even more specific than the general set of standards it conforms to.""")
    created_by: Optional[List[str]] = Field(default_factory=list, description="""Agent that created the element""")
    created_on: Optional[str] = Field(None, description="""Date and Time at which the element was created""")
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    doi: Optional[str] = Field(None, description="""The Digital Object Identifier of the data, with the doi prefix.""")
    download_url: Optional[str] = Field(None, description="""URL from which the data can be downloaded. This is not the same as the landing page, which is a page that describes the dataset. Rather, this URL points directly to the data itself.""")
    id: str = Field(..., description="""the unique name of the dataset""")
    issued: Optional[str] = Field(None)
    keywords: Optional[List[str]] = Field(default_factory=list, description="""Keywords associated with the data. These may be provided by the data creator or assigned later in a manual or automated manner.""")
    language: Optional[str] = Field(None, description="""language in which the information is expressed""")
    last_updated_on: Optional[str] = Field(None, description="""Date and Time at which the element was last updated""")
    license: Optional[str] = Field(None, description="""license for the data""")
    modified_by: Optional[str] = Field(None, description="""agent that modified the element""")
    page: Optional[str] = Field(None)
    publisher: Optional[str] = Field(None)
    status: Optional[str] = Field(None, description="""Status of the element in terms of its maturity or life cycle""")
    title: Optional[str] = Field(None, description="""the official title of the element""")
    version: Optional[str] = Field(None, description="""particular version of schema""")
    was_derived_from: Optional[str] = Field(None, description="""A derivation is a transformation of an entity into another, an update of an entity resulting in a new one, or the construction of a new entity based on a pre-existing entity.@en""")
    
    

class Dataset(Information):
    """
    A single component of related observations and/or information that can be read, manipulated, transformed, and otherwise interpreted.
    """
    bytes: Optional[int] = Field(None, description="""Size of the data in bytes.""")
    dialect: Optional[str] = Field(None)
    encoding: Optional[EncodingEnum] = Field(None, description="""The encoding of the data. This is not the same as the media type. Rather, this is the encoding of the data in a more specific sense, e.g., UTF-8, ASCII, etc.""")
    format: Optional[FormatEnum] = Field(None, description="""The format of the data. This is not the same as the media type. Rather, this is the format of the data in a more specific sense, e.g., CSV, JSON, etc.""")
    hash: Optional[str] = Field(None, description="""The hash representation of the data, e.g., sha256, md5, etc. Subtypes have their own slots.""")
    md5: Optional[str] = Field(None, description="""The md5 hash representation of the data.""")
    media_type: Optional[str] = Field(None, description="""The media type of the data. This is not the same as the format. Rather, this is the media type of the data in a more general sense, e.g., text/csv, application/json, etc., though as it is defined here the media type can be any string.""")
    path: Optional[str] = Field(None)
    sha256: Optional[str] = Field(None, description="""The sha256 hash representation of the data.""")
    purposes: Optional[List[str]] = Field(default_factory=list)
    tasks: Optional[List[str]] = Field(default_factory=list)
    addressing_gaps: Optional[List[str]] = Field(default_factory=list)
    creators: Optional[List[str]] = Field(default_factory=list)
    funders: Optional[List[str]] = Field(default_factory=list)
    subsets: Optional[List[str]] = Field(default_factory=list)
    instances: Optional[List[str]] = Field(default_factory=list)
    anomalies: Optional[List[str]] = Field(default_factory=list)
    external_resources: Optional[List[str]] = Field(default_factory=list)
    confidential_elements: Optional[List[str]] = Field(default_factory=list)
    content_warnings: Optional[List[str]] = Field(default_factory=list)
    subpopulations: Optional[List[str]] = Field(default_factory=list)
    sensitive_elements: Optional[List[str]] = Field(default_factory=list)
    acquisition_methods: Optional[List[str]] = Field(default_factory=list)
    collection_mechanisms: Optional[List[str]] = Field(default_factory=list)
    sampling_strategies: Optional[List[str]] = Field(default_factory=list)
    data_collectors: Optional[List[str]] = Field(default_factory=list)
    collection_timeframes: Optional[List[str]] = Field(default_factory=list)
    ethical_reviews: Optional[List[str]] = Field(default_factory=list)
    data_protection_impacts: Optional[List[str]] = Field(default_factory=list)
    preprocessing_strategies: Optional[List[str]] = Field(default_factory=list)
    cleaning_strategies: Optional[List[str]] = Field(default_factory=list)
    labeling_strategies: Optional[List[str]] = Field(default_factory=list)
    raw_sources: Optional[List[str]] = Field(default_factory=list)
    existing_uses: Optional[List[str]] = Field(default_factory=list)
    use_repository: Optional[List[str]] = Field(default_factory=list)
    other_tasks: Optional[List[str]] = Field(default_factory=list)
    future_use_impacts: Optional[List[str]] = Field(default_factory=list)
    discouraged_uses: Optional[List[str]] = Field(default_factory=list)
    distribution_formats: Optional[List[str]] = Field(default_factory=list)
    distribution_dates: Optional[List[str]] = Field(default_factory=list)
    license_and_use_terms: Optional[str] = Field(None)
    ip_restrictions: Optional[str] = Field(None)
    regulatory_restrictions: Optional[str] = Field(None)
    maintainers: Optional[List[str]] = Field(default_factory=list)
    errata: Optional[List[str]] = Field(default_factory=list)
    updates: Optional[str] = Field(None)
    retention_limit: Optional[str] = Field(None)
    version_access: Optional[str] = Field(None)
    extension_mechanism: Optional[str] = Field(None)
    is_deidentified: Optional[str] = Field(None)
    is_tabular: Optional[str] = Field(None)
    compression: Optional[CompressionEnum] = Field(None, description="""The compression format of the data. This is not the same as the media type. Rather, this is the compression format of the data in a more specific sense, e.g., zip, gzip, etc.""")
    conforms_to: Optional[str] = Field(None, description="""The standard to which the data conforms. This is not the same as the media type. Rather, this is the standard to which the data conforms in a more specific sense, e.g., frictionless, schema.org, etc.""")
    conforms_to_class: Optional[str] = Field(None, description="""The class in the schema to which the data object instantiates.""")
    conforms_to_schema: Optional[str] = Field(None, description="""The schema to which the data conforms. This is not the same as the media type. Rather, this is the schema to which the data conforms in a more specific sense, and even more specific than the general set of standards it conforms to.""")
    created_by: Optional[List[str]] = Field(default_factory=list, description="""Agent that created the element""")
    created_on: Optional[str] = Field(None, description="""Date and Time at which the element was created""")
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    doi: Optional[str] = Field(None, description="""The Digital Object Identifier of the data, with the doi prefix.""")
    download_url: Optional[str] = Field(None, description="""URL from which the data can be downloaded. This is not the same as the landing page, which is a page that describes the dataset. Rather, this URL points directly to the data itself.""")
    id: str = Field(..., description="""the unique name of the dataset""")
    issued: Optional[str] = Field(None)
    keywords: Optional[List[str]] = Field(default_factory=list, description="""Keywords associated with the data. These may be provided by the data creator or assigned later in a manual or automated manner.""")
    language: Optional[str] = Field(None, description="""language in which the information is expressed""")
    last_updated_on: Optional[str] = Field(None, description="""Date and Time at which the element was last updated""")
    license: Optional[str] = Field(None, description="""license for the data""")
    modified_by: Optional[str] = Field(None, description="""agent that modified the element""")
    page: Optional[str] = Field(None)
    publisher: Optional[str] = Field(None)
    status: Optional[str] = Field(None, description="""Status of the element in terms of its maturity or life cycle""")
    title: Optional[str] = Field(None, description="""the official title of the element""")
    version: Optional[str] = Field(None, description="""particular version of schema""")
    was_derived_from: Optional[str] = Field(None, description="""A derivation is a transformation of an entity into another, an update of an entity resulting in a new one, or the construction of a new entity based on a pre-existing entity.@en""")
    
    

class DataSubset(Dataset):
    """
    A subset of a dataset, likely containing multiple files of multiple potential purposes and properties.
    """
    is_data_split: Optional[str] = Field(None, description="""Is this subset a split of the larger dataset, e.g., is it a set for model training, testing, or validation?""")
    is_subpopulation: Optional[str] = Field(None, description="""Is this subset a subpopulation of the larger dataset, e.g., is it a set of data for a specific demographic?""")
    bytes: Optional[int] = Field(None, description="""Size of the data in bytes.""")
    dialect: Optional[str] = Field(None)
    encoding: Optional[EncodingEnum] = Field(None, description="""The encoding of the data. This is not the same as the media type. Rather, this is the encoding of the data in a more specific sense, e.g., UTF-8, ASCII, etc.""")
    format: Optional[FormatEnum] = Field(None, description="""The format of the data. This is not the same as the media type. Rather, this is the format of the data in a more specific sense, e.g., CSV, JSON, etc.""")
    hash: Optional[str] = Field(None, description="""The hash representation of the data, e.g., sha256, md5, etc. Subtypes have their own slots.""")
    md5: Optional[str] = Field(None, description="""The md5 hash representation of the data.""")
    media_type: Optional[str] = Field(None, description="""The media type of the data. This is not the same as the format. Rather, this is the media type of the data in a more general sense, e.g., text/csv, application/json, etc., though as it is defined here the media type can be any string.""")
    path: Optional[str] = Field(None)
    sha256: Optional[str] = Field(None, description="""The sha256 hash representation of the data.""")
    purposes: Optional[List[str]] = Field(default_factory=list)
    tasks: Optional[List[str]] = Field(default_factory=list)
    addressing_gaps: Optional[List[str]] = Field(default_factory=list)
    creators: Optional[List[str]] = Field(default_factory=list)
    funders: Optional[List[str]] = Field(default_factory=list)
    subsets: Optional[List[str]] = Field(default_factory=list)
    instances: Optional[List[str]] = Field(default_factory=list)
    anomalies: Optional[List[str]] = Field(default_factory=list)
    external_resources: Optional[List[str]] = Field(default_factory=list)
    confidential_elements: Optional[List[str]] = Field(default_factory=list)
    content_warnings: Optional[List[str]] = Field(default_factory=list)
    subpopulations: Optional[List[str]] = Field(default_factory=list)
    sensitive_elements: Optional[List[str]] = Field(default_factory=list)
    acquisition_methods: Optional[List[str]] = Field(default_factory=list)
    collection_mechanisms: Optional[List[str]] = Field(default_factory=list)
    sampling_strategies: Optional[List[str]] = Field(default_factory=list)
    data_collectors: Optional[List[str]] = Field(default_factory=list)
    collection_timeframes: Optional[List[str]] = Field(default_factory=list)
    ethical_reviews: Optional[List[str]] = Field(default_factory=list)
    data_protection_impacts: Optional[List[str]] = Field(default_factory=list)
    preprocessing_strategies: Optional[List[str]] = Field(default_factory=list)
    cleaning_strategies: Optional[List[str]] = Field(default_factory=list)
    labeling_strategies: Optional[List[str]] = Field(default_factory=list)
    raw_sources: Optional[List[str]] = Field(default_factory=list)
    existing_uses: Optional[List[str]] = Field(default_factory=list)
    use_repository: Optional[List[str]] = Field(default_factory=list)
    other_tasks: Optional[List[str]] = Field(default_factory=list)
    future_use_impacts: Optional[List[str]] = Field(default_factory=list)
    discouraged_uses: Optional[List[str]] = Field(default_factory=list)
    distribution_formats: Optional[List[str]] = Field(default_factory=list)
    distribution_dates: Optional[List[str]] = Field(default_factory=list)
    license_and_use_terms: Optional[str] = Field(None)
    ip_restrictions: Optional[str] = Field(None)
    regulatory_restrictions: Optional[str] = Field(None)
    maintainers: Optional[List[str]] = Field(default_factory=list)
    errata: Optional[List[str]] = Field(default_factory=list)
    updates: Optional[str] = Field(None)
    retention_limit: Optional[str] = Field(None)
    version_access: Optional[str] = Field(None)
    extension_mechanism: Optional[str] = Field(None)
    is_deidentified: Optional[str] = Field(None)
    is_tabular: Optional[str] = Field(None)
    compression: Optional[CompressionEnum] = Field(None, description="""The compression format of the data. This is not the same as the media type. Rather, this is the compression format of the data in a more specific sense, e.g., zip, gzip, etc.""")
    conforms_to: Optional[str] = Field(None, description="""The standard to which the data conforms. This is not the same as the media type. Rather, this is the standard to which the data conforms in a more specific sense, e.g., frictionless, schema.org, etc.""")
    conforms_to_class: Optional[str] = Field(None, description="""The class in the schema to which the data object instantiates.""")
    conforms_to_schema: Optional[str] = Field(None, description="""The schema to which the data conforms. This is not the same as the media type. Rather, this is the schema to which the data conforms in a more specific sense, and even more specific than the general set of standards it conforms to.""")
    created_by: Optional[List[str]] = Field(default_factory=list, description="""Agent that created the element""")
    created_on: Optional[str] = Field(None, description="""Date and Time at which the element was created""")
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    doi: Optional[str] = Field(None, description="""The Digital Object Identifier of the data, with the doi prefix.""")
    download_url: Optional[str] = Field(None, description="""URL from which the data can be downloaded. This is not the same as the landing page, which is a page that describes the dataset. Rather, this URL points directly to the data itself.""")
    id: str = Field(..., description="""the unique name of the dataset""")
    issued: Optional[str] = Field(None)
    keywords: Optional[List[str]] = Field(default_factory=list, description="""Keywords associated with the data. These may be provided by the data creator or assigned later in a manual or automated manner.""")
    language: Optional[str] = Field(None, description="""language in which the information is expressed""")
    last_updated_on: Optional[str] = Field(None, description="""Date and Time at which the element was last updated""")
    license: Optional[str] = Field(None, description="""license for the data""")
    modified_by: Optional[str] = Field(None, description="""agent that modified the element""")
    page: Optional[str] = Field(None)
    publisher: Optional[str] = Field(None)
    status: Optional[str] = Field(None, description="""Status of the element in terms of its maturity or life cycle""")
    title: Optional[str] = Field(None, description="""the official title of the element""")
    version: Optional[str] = Field(None, description="""particular version of schema""")
    was_derived_from: Optional[str] = Field(None, description="""A derivation is a transformation of an entity into another, an update of an entity resulting in a new one, or the construction of a new entity based on a pre-existing entity.@en""")
    
    

class Software(NamedThing):
    """
    A software program or library.
    """
    version: Optional[str] = Field(None)
    license: Optional[str] = Field(None)
    url: Optional[str] = Field(None)
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class Purpose(DatasetProperty):
    """
    For what purpose was the dataset created?
    """
    response: Optional[str] = Field(None)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class Task(DatasetProperty):
    """
    Was there a specific task in mind for the dataset's application?
    """
    response: Optional[str] = Field(None)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class AddressingGap(DatasetProperty):
    """
    Was there a specific gap that needed to be filled by creation of the dataset?
    """
    response: Optional[str] = Field(None)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class Creator(DatasetProperty):
    """
    Who created the dataset (e.g., which team, research group) and on behalf of which entity (e.g., company, institution, organization)? This may also be considered a team.
    """
    principal_investigator: Optional[str] = Field(None)
    affiliation: Optional[str] = Field(None)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class FundingMechanism(DatasetProperty):
    """
    Who funded the creation of the dataset? If there is an associated grant, please provide the name of the grantor and the grant name and number.
    """
    grantor: Optional[str] = Field(None)
    grant: Optional[str] = Field(None)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class Grantor(Organization):
    """
    What is the name and/or identifier of the organization providing monetary support or other resources supporting creation of the dataset?
    """
    email: Optional[str] = Field(None, description="""The email address of the organization.""")
    ror_id: Optional[str] = Field(None, description="""Unique ROR identifier.""")
    wikidata_id: Optional[str] = Field(None, description="""Unique Wikidata identifier.""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class Grant(NamedThing):
    """
    What is the name and/or identifier of the specific mechanism providing monetary support or other resources supporting creation of the dataset?
    """
    grant_number: Optional[str] = Field(None, description="""The alphanumeric identifier for the grant.""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class Instance(DatasetProperty):
    """
    What do the instances that comprise the dataset represent (e.g., documents, photos, people, countries)?
    """
    representation: Optional[str] = Field(None)
    instance_type: Optional[str] = Field(None, description="""Are there multiple types of instances (e.g., movies, users, and ratings; people and interactions between them; nodes and edges)?""")
    data_type: Optional[str] = Field(None, description="""What data does each instance consist of? “Raw” data (e.g., unprocessed text or images) or features? In either case, please provide a description.""")
    counts: Optional[int] = Field(None, description="""How many instances are there in total (of each type, if appropriate)?""")
    label: Optional[str] = Field(None, description="""Is there a label or target associated with each instance?""")
    sampling_strategies: Optional[List[str]] = Field(default_factory=list)
    missing_information: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class SamplingStrategy(DatasetProperty):
    """
    Does the dataset contain all possible instances or is it a sample (not necessarily random) of instances from a larger set? If the dataset is a sample, then what is the larger set? Is the sample representative of the larger set (e.g., geographic coverage)? If so, please describe how this representativeness was validated/verified. If it is not representative of the larger set, please describe why not (e.g., to cover a more diverse range of instances, because instances were withheld or unavailable).
    """
    is_sample: Optional[List[str]] = Field(default_factory=list)
    is_random: Optional[List[str]] = Field(default_factory=list)
    source_data: Optional[List[str]] = Field(default_factory=list)
    is_representative: Optional[List[str]] = Field(default_factory=list)
    representative_verification: Optional[List[str]] = Field(default_factory=list)
    why_not_representative: Optional[List[str]] = Field(default_factory=list)
    strategies: Optional[List[str]] = Field(default_factory=list, description="""If the dataset is a sample from a larger set, what was the sampling strategy (e.g., deterministic, probabilistic with specific sampling probabilities)?""")
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class MissingInfo(DatasetProperty):
    """
    Is any information missing from individual instances? If so, please provide a description, explaining why this information is missing (e.g., because it was unavailable). This does not include intentionally removed information, but might include, e.g., redacted text.
    """
    missing: Optional[List[str]] = Field(default_factory=list)
    why_missing: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class Relationships(DatasetProperty):
    """
    Are relationships between individual instances made explicit (e.g., users’ movie ratings, social network links)? If so, please describe how these relationships are made explicit.
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class Splits(DatasetProperty):
    """
    Are there recommended data splits (e.g., training, development/validation, testing)? If so, please provide a description of these splits, explaining the rationale behind them.
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class DataAnomaly(DatasetProperty):
    """
    Are there any errors, sources of noise, or redundancies in the dataset? If so, please provide a description.
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class ExternalResource(DatasetProperty):
    """
    Is the dataset self-contained, or does it link to or otherwise rely on external resources (e.g., websites, tweets, other datasets)? If it links to or relies on external resources, a) are there guarantees that they will exist, and remain constant, over time; b) are there official archival versions of the complete dataset (i.e., including the external resources as they existed at the time the dataset was created); c) are there any restrictions (e.g., licenses, fees) associated with any of the external resources that might apply to a dataset consumer? Please provide descriptions of all external resources and any restrictions associated with them, as well as links or other access points, as appropriate.
    """
    external_resources: Optional[List[str]] = Field(default_factory=list)
    future_guarantees: Optional[List[str]] = Field(default_factory=list)
    archival: Optional[List[str]] = Field(default_factory=list)
    restrictions: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class Confidentiality(DatasetProperty):
    """
    Does the dataset contain data that might be considered confidential (e.g., data that is protected by legal privilege or by doctor patient confidentiality, data that includes the content of individuals’ non-public communications)?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class ContentWarning(DatasetProperty):
    """
    Does the dataset contain data that, if viewed directly, might be offensive, insulting, threatening, or might otherwise cause anxiety? If so, please describe why.
    """
    warnings: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class Subpopulation(DatasetProperty):
    """
    Does the dataset identify any subpopulations (e.g., by age, gender)? If so, please describe how these subpopulations are identified and provide a description of their respective distributions within the dataset.
    """
    identification: Optional[List[str]] = Field(default_factory=list)
    distribution: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    description: Optional[str] = Field(None, description="""human readable description of the information""")
    
    

class Deidentification(DatasetProperty):
    """
    Is it possible to identify individuals (i.e., one or more natural persons), either directly or indirectly (i.e., in combination with other data) from the dataset?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class SensitiveElement(DatasetProperty):
    """
    Does the dataset contain data that might be considered sensitive in any way (e.g., data that reveals race or ethnic origins, sexual orientations, religious beliefs, political opinions or union memberships, or locations; financial or health data; biometric or genetic data; forms of government identification, such as social security numbers; criminal history)?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class InstanceAcquisition(DatasetProperty):
    """
    How was the data associated with each instance acquired? Was the data directly observable (e.g., raw text, movie ratings), reported by subjects (e.g., survey responses), or indirectly inferred/derived from other data (e.g., part-of-speech tags, model-based guesses for age or language)? If the data was reported by subjects or indirectly inferred/derived from other data, was the data validated/verified?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    was_directly_observed: Optional[str] = Field(None, description="""Was the data directly observable (e.g., raw text, movie ratings)?""")
    was_reported_by_subjects: Optional[str] = Field(None, description="""Was the data reported by subjects (e.g., survey responses)?""")
    was_inferred_derived: Optional[str] = Field(None, description="""Was the data indirectly inferred/derived from other data (e.g., part-of-speech tags, model-based guesses for age or language)?""")
    was_validated_verified: Optional[str] = Field(None, description="""Was the data validated/verified?""")
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class CollectionMechanism(DatasetProperty):
    """
    What mechanisms or procedures were used to collect the data (e.g., hardware apparatuses or sensors, manual human curation, software programs, software APIs)? How were these mechanisms or procedures validated?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class DataCollector(DatasetProperty):
    """
    Who was involved in the data collection process (e.g., students, crowdworkers, contractors) and how were they compensated (e.g., how much were crowdworkers paid)?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class CollectionTimeframe(DatasetProperty):
    """
    Over what timeframe was the data collected? Does this timeframe match the creation timeframe of the data associated with the instances (e.g., recent crawl of old news articles)? If not, please describe the timeframe in which the data associated with the instances was created.
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class EthicalReview(DatasetProperty):
    """
    Were any ethical review processes conducted (e.g., by an institutional review board)? If so, please provide a description of these review processes, including the outcomes, as well as a link or other access point to any supporting documentation.
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class DirectCollection(DatasetProperty):
    """
    Did you collect the data from the individuals in question directly, or obtain it via third parties or other sources (e.g., websites)?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class CollectionNotification(DatasetProperty):
    """
    Were the individuals in question notified about the data collection? If so, please describe (or show with screenshots or other information) how notice was provided, and provide a link or other access point to, or otherwise reproduce, the exact language of the notification itself.
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class CollectionConsent(DatasetProperty):
    """
    Did the individuals in question consent to the collection and use of their data? If so, please describe (or show with screenshots or other information) how consent was requested and provided, and provide a link or other access point to, or otherwise reproduce, the exact language to which the individuals consented.
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class ConsentRevocation(DatasetProperty):
    """
    If consent was obtained, were the consenting individuals provided with a mechanism to revoke their consent in the future or 8 for certain uses? If so, please provide a description, as well as a link or other access point to the mechanism (if appropriate).
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class DataProtectionImpact(DatasetProperty):
    """
    Has an analysis of the potential impact of the dataset and its use on data subjects (e.g., a data protection impact analysis) been conducted? If so, please provide a description of this analysis, including the outcomes, as well as a link or other access point to any supporting documentation.
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class PreprocessingStrategy(DatasetProperty):
    """
    Was any preprocessing of the data done (e.g., discretization or bucketing, tokenization, SIFT feature extraction)?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class CleaningStrategy(DatasetProperty):
    """
    Was any cleaning of the data done (e.g., removal of instances, processing of missing values)?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class LabelingStrategy(DatasetProperty):
    """
    Was any preprocessing/cleaning/labeling of the data done (e.g., part-of-speech tagging)?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class RawData(DatasetProperty):
    """
    Was the “raw” data saved in addition to the preprocessed/cleaned/labeled data (e.g., to support unanticipated future uses)? If so, please provide a link or other access point to the “raw” data.
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class ExistingUse(DatasetProperty):
    """
    Has the dataset been used for any tasks already?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class UseRepository(DatasetProperty):
    """
    Is there a repository that links to any or all papers or systems that use the dataset? If so, please provide a link or other access point.
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class OtherTask(DatasetProperty):
    """
    What (other) tasks could the dataset be used for?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class FutureUseImpact(DatasetProperty):
    """
    Is there anything about the composition of the dataset or the way it was collected and preprocessed/cleaned/labeled that might impact future uses? For example, is there anything that a dataset consumer might need to know to avoid uses that could result in unfair treatment of individuals or groups (e.g., stereotyping, quality of service issues) or other risks or harms (e.g., legal risks, financial harms)? If so, please provide a description. Is there anything a dataset consumer could do to mitigate these risks or harms?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class DiscouragedUse(DatasetProperty):
    """
    Are there tasks for which the dataset should not be used?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class ThirdPartySharing(DatasetProperty):
    """
    Will the dataset be distributed to third parties outside of the entity (e.g., company, institution, organization) on behalf of which the dataset was created?
    """
    description: Optional[str] = Field(None)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class DistributionFormat(DatasetProperty):
    """
    How will the dataset will be distributed (e.g., tarball on website, API, GitHub)?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class DistributionDate(DatasetProperty):
    """
    When will the dataset be distributed?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class LicenseAndUseTerms(DatasetProperty):
    """
    Will the dataset be distributed under a copyright or other intellectual property (IP) license, and/or under applicable terms of use (ToU)? If so, please describe this license and/or ToU, and provide a link or other access point to, or otherwise reproduce, any relevant licensing terms or ToU, as well as any fees associated with these restrictions.
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class IPRestrictions(DatasetProperty):
    """
    Have any third parties imposed IP-based or other restrictions on the data associated with the instances? If so, please describe these restrictions, and provide a link or other access point to, or otherwise reproduce, any relevant licensing terms, as well as any fees associated with these restrictions.
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class ExportControlRegulatoryRestrictions(DatasetProperty):
    """
    Do any export controls or other regulatory restrictions apply to the dataset or to individual instances? If so, please describe these restrictions, and provide a link or other access point to, or otherwise reproduce, any supporting documentation.
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class Maintainer(DatasetProperty):
    """
    Who will be supporting/hosting/maintaining the dataset?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class Erratum(DatasetProperty):
    """
    Is there an erratum? If so, please provide a link or other access point.
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class UpdatePlan(DatasetProperty):
    """
    Will the dataset be updated (e.g., to correct labeling errors, add new instances, delete instances)? If so, please describe how often, by whom, and how updates will be communicated to dataset consumers (e.g., mailing list, GitHub)?
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class RetentionLimits(DatasetProperty):
    """
    If the dataset relates to people, are there applicable limits on the retention of the data associated with the instances (e.g., were the individuals in question told that their data would be retained for a fixed period of time and then deleted)? If so, please describe these limits and explain how they will be enforced.
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class VersionAccess(DatasetProperty):
    """
    Will older versions of the dataset continue to be supported/hosted/maintained? If so, please describe how. If not, please describe how its obsolescence will be communicated to dataset consumers.
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    

class ExtensionMechanism(DatasetProperty):
    """
    If others want to extend/augment/build on/contribute to the dataset, is there a mechanism for them to do so? If so, please provide a description. Will these contributions be validated/verified? If so, please describe how. If not, why not? Is there a process for communicating/distributing these contributions to dataset consumers? If so, please provide a description.
    """
    description: Optional[List[str]] = Field(default_factory=list)
    used_software: Optional[List[str]] = Field(default_factory=list, description="""What software was used as part of this dataset property?""")
    id: str = Field(..., description="""the unique name of the dataset""")
    name: Optional[str] = Field(None)
    
    


# Model rebuild
# see https://pydantic-docs.helpmanual.io/usage/models/#rebuilding-a-model
NamedThing.model_rebuild()
Information.model_rebuild()
FormatDialect.model_rebuild()
Person.model_rebuild()
Organization.model_rebuild()
DatasetProperty.model_rebuild()
DatasetCollection.model_rebuild()
Dataset.model_rebuild()
DataSubset.model_rebuild()
Software.model_rebuild()
Purpose.model_rebuild()
Task.model_rebuild()
AddressingGap.model_rebuild()
Creator.model_rebuild()
FundingMechanism.model_rebuild()
Grantor.model_rebuild()
Grant.model_rebuild()
Instance.model_rebuild()
SamplingStrategy.model_rebuild()
MissingInfo.model_rebuild()
Relationships.model_rebuild()
Splits.model_rebuild()
DataAnomaly.model_rebuild()
ExternalResource.model_rebuild()
Confidentiality.model_rebuild()
ContentWarning.model_rebuild()
Subpopulation.model_rebuild()
Deidentification.model_rebuild()
SensitiveElement.model_rebuild()
InstanceAcquisition.model_rebuild()
CollectionMechanism.model_rebuild()
DataCollector.model_rebuild()
CollectionTimeframe.model_rebuild()
EthicalReview.model_rebuild()
DirectCollection.model_rebuild()
CollectionNotification.model_rebuild()
CollectionConsent.model_rebuild()
ConsentRevocation.model_rebuild()
DataProtectionImpact.model_rebuild()
PreprocessingStrategy.model_rebuild()
CleaningStrategy.model_rebuild()
LabelingStrategy.model_rebuild()
RawData.model_rebuild()
ExistingUse.model_rebuild()
UseRepository.model_rebuild()
OtherTask.model_rebuild()
FutureUseImpact.model_rebuild()
DiscouragedUse.model_rebuild()
ThirdPartySharing.model_rebuild()
DistributionFormat.model_rebuild()
DistributionDate.model_rebuild()
LicenseAndUseTerms.model_rebuild()
IPRestrictions.model_rebuild()
ExportControlRegulatoryRestrictions.model_rebuild()
Maintainer.model_rebuild()
Erratum.model_rebuild()
UpdatePlan.model_rebuild()
RetentionLimits.model_rebuild()
VersionAccess.model_rebuild()
ExtensionMechanism.model_rebuild()




"""
Evaluation Engines.

An evaluation engine incorporates different components to evaluate KE:

 1. ETL-ing the data from the source and mapping to a target schema
 2. Selecting test and training cases from a source database
 3. Executing the KE
 4. Calculating scores comparing test with training

"""

from dataclasses import dataclass
from typing import List, Optional, Set, Type, Union 

from oaklib import BasicOntologyInterface
from pydantic import BaseModel

from ontogpt.engines.spires_engine import SPIRESEngine


def jaccard_index(a: Set, b: Set):
    """Compute the Jaccard index between two sets."""
    if not a and not b:
        return None
    return len(a & b) / len(a | b)


class SimilarityScore(BaseModel):
    """
    Scores for an individual prediction.

    The scores are computed as the Jaccard index between the predicted and true sets.
    """

    jaccard: Optional[float]
    false_positives: Optional[List[Optional[str]]]
    false_negatives: Optional[List[Optional[str]]]
    common: Optional[List[str]]

    @staticmethod
    def from_set(
        test_list: List,
        prediction_list: List,
        labelers: Optional[List[BasicOntologyInterface]] = None,
    ):
        if labelers:

            def label(x):
                for labeler in labelers:
                    if type(labeler) == list:
                        lbl = labeler[0].label(x)
                    else:
                        lbl = labeler.label(x)
                    if lbl:
                        return f"{x} {lbl}"
                return x

            test_list = [label(x) for x in test_list]
            prediction_list = [label(x) for x in prediction_list]
        test_set = set(test_list)
        prediction_set = set(prediction_list)
        return SimilarityScore(
            jaccard=jaccard_index(test_set, prediction_set),
            false_positives=list(prediction_set - test_set),
            false_negatives=list(test_set - prediction_set),
            common=list(test_set & prediction_set),
        )


@dataclass
class EvaluationEngine:
    pass


@dataclass
class SPIRESEvaluationEngine(EvaluationEngine):
    """Base class for all evaluation engines in SPIRES.

    TODO: currently there is a lot of repetition between the different evaluation engines.
    This should be refactored so we have common base classes for different tasks

    - NamedEntity Recognition
    - Simple Relation Extraction, i.e. pairs of subject-object (e.g CTD task)
    - Triple Extraction, i.e. triples of subject-predicate-object
    - Complex Relation Extraction, i.e. triples of subject-predicate-object with\
        additional information, e.g. qualifiers
    - Structured Knowledge Extraction, i.e. arbitrary nested objects including\
        both literals and edges
    """

    extractor: SPIRESEngine = None
    """Knowledge extractor to use"""

    num_tests: Optional[Union[int, Type]] = 10
    """Number of test cases to use for evaluation"""

    num_training: Optional[Union[int, Type]] = 5
    """Number of training/exemplar cases to use for evaluation in generalization task.
    Note this number will be low as we use few-shot learning."""

    chunking: Optional[Union[bool, Type]] = False
    """Whether to pre-process input texts by chunking. If True, each chunk gets its own
    prompt. Otherwise, pass the full text with each prompt."""

    model: Optional[Union[str, Type]] = None
    """Name of the model to use in evaluation."""



"""Synonym engine."""
import logging
from copy import deepcopy
from dataclasses import dataclass
from pathlib import Path
from typing import Iterator, List, Union

from jinja2 import Template
from pydantic import BaseModel

from ontogpt.engines.knowledge_engine import KnowledgeEngine
from ontogpt.prompts.qa import GENERIC_QA_PROMPT

logger = logging.getLogger(__name__)

MAX_TOKENS = 300


class Question(BaseModel):
    name: str
    question: str
    original_question: str = ""
    answer: str = ""
    accepted_answer: str = ""
    iteration: int = 0
    instructions: str = ""
    prompt: str = ""
    model: str = ""


class Instruction(BaseModel):
    name: str
    text: str = ""
    template: str = ""


class QuestionCollection(BaseModel):
    url: str = ""
    questions: List[Question] = []
    instructions: List[Instruction] = []


@dataclass
class GenericEngine(KnowledgeEngine):
    def run(
        self,
        question_collection: QuestionCollection,
        template_path: Union[str, Path] = "",
    ) -> Iterator[Question]:
        if template_path is None:
            template_path = GENERIC_QA_PROMPT
        if isinstance(template_path, Path):
            template_path = str(template_path)
        if isinstance(template_path, str):
            # create a Jinja2 template object
            with open(template_path) as file:
                template_txt = file.read()
                main_template = Template(template_txt)
        for question in question_collection.questions:
            for instruction in question_collection.instructions:
                template = main_template
                if instruction.template is not None:
                    template = Template(instruction.template)
                prompt = template.render(
                    question=question.question,
                    instructions=instruction.text,
                )
                payload = self.client.complete(prompt, max_tokens=MAX_TOKENS)
                question_with_answer = deepcopy(question)
                question_with_answer.answer = payload
                question_with_answer.instructions = instruction.name
                question_with_answer.prompt = prompt
                question_with_answer.model = self.model
                yield question_with_answer



"""Utilities for processing PDF files using PyMuPDF."""

from operator import itemgetter


def headers_para(doc, size_tag):
    """Scrapes headers & paragraphs from PDF and return texts with element tags.

    :param doc: PDF document to iterate through
    :type doc: <class 'fitz.fitz.Document'>
    :param size_tag: textual element tags for each size
    :type size_tag: dict
    :rtype: list
    :return: texts with pre-prended element tags
    """
    header_para = []  # list with headers and paragraphs
    first = True  # boolean operator for first header
    previous_s = {}  # previous span

    for page in doc:
        blocks = page.get_text("dict")["blocks"]
        for b in blocks:  # iterate through the text blocks
            if b["type"] == 0:  # this block contains text

                # REMEMBER: multiple fonts and sizes are possible IN one block

                block_string = ""  # text found in block
                for line in b["lines"]:  # iterate through the text lines
                    for span in line["spans"]:  # iterate through the text spans
                        if span["text"].strip():  # removing whitespaces:
                            if first:
                                previous_s = span
                                first = False
                                block_string = size_tag[span["size"]] + span["text"]
                            else:
                                if span["size"] == previous_s["size"]:

                                    if block_string and all((c == "|") for c in block_string):
                                        # block_string only contains pipes
                                        block_string = size_tag[span["size"]] + span["text"]
                                    if block_string == "":
                                        # new block has started, so append size tag
                                        block_string = size_tag[span["size"]] + span["text"]
                                    else:  # in the same block, so concatenate strings
                                        block_string += " " + span["text"]

                                else:
                                    header_para.append(block_string)
                                    block_string = size_tag[span["size"]] + span["text"]

                                previous_s = span

                    # new block started, indicating with a pipe
                    block_string += "|"

                header_para.append(block_string)

    return header_para


def fonts(doc, granularity=False):
    """Extract fonts and their usage in PDF documents.

    :param doc: PDF document to iterate through
    :type doc: <class 'fitz.fitz.Document'>
    :param granularity: also use 'font', 'flags' and 'color' to discriminate text
    :type granularity: bool
    :rtype: [(font_size, count), (font_size, count}], dict
    :return: most used fonts sorted by count, font style information
    """
    styles = {}
    font_counts = {}

    for page in doc:
        blocks = page.get_text("dict")["blocks"]
        for b in blocks:  # iterate through the text blocks
            if b["type"] == 0:  # block contains text
                for line in b["lines"]:  # iterate through the text lines
                    for span in line["spans"]:  # iterate through the text spans
                        if granularity:
                            identifier = "{0}_{1}_{2}_{3}".format(
                                span["size"], span["flags"], span["font"], span["color"]
                            )
                            styles[identifier] = {
                                "size": span["size"],
                                "flags": span["flags"],
                                "font": span["font"],
                                "color": span["color"],
                            }
                        else:
                            identifier = "{0}".format(span["size"])
                            styles[identifier] = {"size": span["size"], "font": span["font"]}

                        font_counts[identifier] = (
                            font_counts.get(identifier, 0) + 1
                        )  # count the fonts usage

    font_counts = sorted(font_counts.items(), key=itemgetter(1), reverse=True)

    if len(font_counts) < 1:
        raise ValueError("Zero discriminating fonts found!")

    return font_counts, styles


def font_tags(font_counts, styles):
    """Return dictionary with font sizes as keys and tags as value.

    :param font_counts: (font_size, count) for all fonts occuring in document
    :type font_counts: list
    :param styles: all styles found in the document
    :type styles: dict

    :rtype: dict
    :return: all element tags based on font-sizes
    """
    p_style = styles[font_counts[0][0]]  # get style for most used font by count (paragraph)
    p_size = p_style["size"]  # get the paragraph's size

    # sorting the font sizes high to low, so that we can append the right integer to each tag
    font_sizes = []
    for font_size, _count in font_counts:
        font_sizes.append(float(font_size))
    font_sizes.sort(reverse=True)

    # aggregating the tags for each font size
    idx = 0
    size_tag = {}
    for size in font_sizes:
        idx += 1
        if size == p_size:
            idx = 0
            size_tag[size] = "<p>"
        if size > p_size:
            size_tag[size] = "<h{0}>".format(idx)
        elif size < p_size:
            size_tag[size] = "<s{0}>".format(idx)

    return size_tag



from __future__ import annotations
from datetime import datetime, date
from enum import Enum

from typing import List, Dict, Optional, Any, Union
from pydantic import BaseModel as BaseModel, ConfigDict,  Field, field_validator
import re
import sys
if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal


metamodel_version = "None"
version = "None"

class ConfiguredBaseModel(BaseModel):
    model_config = ConfigDict(
        validate_assignment=True,
        validate_default=True,
        extra = 'forbid',
        arbitrary_types_allowed=True,
        use_enum_values = True)
    pass


class NullDataOptions(str, Enum):
    
    
    UNSPECIFIED_METHOD_OF_ADMINISTRATION = "UNSPECIFIED_METHOD_OF_ADMINISTRATION"
    
    NOT_APPLICABLE = "NOT_APPLICABLE"
    
    NOT_MENTIONED = "NOT_MENTIONED"
    
    

class ClinicalObservationSet(ConfiguredBaseModel):
    """
    A set of sets of clinical observations.
    """
    observations: Optional[List[str]] = Field(default_factory=list)
    
    

class MalnutritionObservations(ConfiguredBaseModel):
    
    malnutrition_presence: Optional[str] = Field(None, description="""True if the patient is malnourished, False otherwise. N/A if not provided.""")
    malnutrition_risk: Optional[str] = Field(None, description="""True if the patient has a demonstrable risk for malnutrition, False otherwise. N/A if not provided.""")
    severity: Optional[str] = Field(None, description="""The severity of the patient's malnutrition, if present. This may be Mild, Moderate, or Severe. In general, a patient receiving less than 50% of their estimated energy requirement for greater than 5 days is considered to have severe malnutrition. N/A if not provided.""")
    acute_or_chronic: Optional[str] = Field(None, description="""The duration of the patient's malnutrition, if present. For pediatric patients, acute malnutrition is less than 3 months, and chronic malnutrition is greater than 3 months. This may be Acute or Chronic. N/A if not provided.""")
    diagnosis: Optional[str] = Field(None, description="""The patient's malnutrition diagnosis, if present. This should not include modifiers like 'severe'. N/A if not provided.""")
    etiology: Optional[str] = Field(None, description="""The cause of the patient's malnutrition, if known. This may be due to acute or chronic disease or social/behavioral factors. N/A if not provided.""")
    risk_for_refeeding_syndrome: Optional[str] = Field(None, description="""True if the patient is at risk for refeeding syndrome, False otherwise. N/A if not provided.""")
    
    

class ExtractionResult(ConfiguredBaseModel):
    """
    A result of extracting knowledge on text
    """
    input_id: Optional[str] = Field(None)
    input_title: Optional[str] = Field(None)
    input_text: Optional[str] = Field(None)
    raw_completion_output: Optional[str] = Field(None)
    prompt: Optional[str] = Field(None)
    extracted_object: Optional[Any] = Field(None, description="""The complex objects extracted from the text""")
    named_entities: Optional[List[Any]] = Field(default_factory=list, description="""Named entities extracted from the text""")
    
    

class NamedEntity(ConfiguredBaseModel):
    
    id: str = Field(..., description="""A unique identifier for the named entity""")
    label: Optional[str] = Field(None, description="""The label (name) of the named thing""")
    
    

class ClinicalObservations(NamedEntity):
    """
    A set of clinical observations about a single patient at a single time.
    """
    is_pediatric: Optional[str] = Field(None)
    is_preterm: Optional[str] = Field(None)
    patient_age: Optional[QuantitativeValue] = Field(None)
    patient_height: Optional[QuantitativeValueWithMetric] = Field(None)
    current_patient_weight: Optional[QuantitativeValueWithMetric] = Field(None)
    usual_patient_weight: Optional[QuantitativeValueWithMetric] = Field(None)
    head_circumference: Optional[QuantitativeValueWithMetric] = Field(None)
    malnutrition_status: Optional[MalnutritionObservations] = Field(None)
    diet_supplementation: Optional[List[DietSupplementation]] = Field(default_factory=list, description="""A semicolon-separated list of the patient's diet supplementation therapies. Split on specific ingredients and their amounts. All acronyms should be expanded, omitting the original acronym. Relevant acronyms: PO: per os/by mouth, TPN: total parenteral nutrition, PN: parenteral nutrition, EN: enteral nutrition, D#%: dextrose percentage (e.g. D5%) for PN infusion, AA # g/kg/d: amino acid provisions (may also be in percentages) for PN infusion, SMOF # g/kg/d: soy MCT olive fish oil emulsion for PN infusion, GIR: glucose infusion rate, SBS: short bowel syndrome, LIS: low intermittent suction, BW: birth weight, EHM: exclusively human milk, RTBW: return to birth weight, Mg: magnesium, Phos: phosphorus, GI: gastrointestinal, PICC: peripherally inserted central catheter, DOL: day of life, TG: triglycerides, KUB: Kidney ureter bladder CT""")
    nutrition_support: Optional[List[NutritionSupport]] = Field(default_factory=list, description="""A semicolon-separated list of the patient's nutrition support therapies, usually enteral or parenteral nutrition. All acronyms should be expanded, omitting the original acronym. Relevant acronyms: PO: per os/by mouth, TPN: total parenteral nutrition, PN: parenteral nutrition, EN: enteral nutrition, D#%: dextrose percentage (e.g. D5%) for PN infusion, AA # g/kg/d: amino acid provisions (may also be in percentages) for PN infusion, SMOF # g/kg/d: soy MCT olive fish oil emulsion for PN infusion, GIR: glucose infusion rate, SBS: short bowel syndrome, LIS: low intermittent suction, BW: birth weight, EHM: exclusively human milk, RTBW: return to birth weight, Mg: magnesium, Phos: phosphorus, GI: gastrointestinal, PICC: peripherally inserted central catheter, DOL: day of life, TG: triglycerides, KUB: Kidney ureter bladder CT""")
    medications: Optional[List[DrugTherapy]] = Field(default_factory=list, description="""A semicolon-separated list of the patient's medications. This should include the medication name, dosage, frequency, and route of administration. Relevant acronyms: PO: per os/by mouth, PRN: pro re nata/as needed. 'Not provided' if not provided.""")
    nil_per_os: Optional[str] = Field(None, description="""True if the patient is not receiving any oral nutrition, False otherwise. NPO means nil per os/nothing by mouth. N/A if not indicated.""")
    id: str = Field(..., description="""A unique identifier for the named entity""")
    label: Optional[str] = Field(None, description="""The label (name) of the named thing""")
    
    

class TherapeuticMaterial(NamedEntity):
    """
    A specific material added to a patient's diet or included as part of a nutritional plan.
    """
    id: str = Field(..., description="""A unique identifier for the named entity""")
    label: Optional[str] = Field(None, description="""The label (name) of the named thing""")
    
    

class NutritionSupportMethod(NamedEntity):
    """
    A method of nutrition support therapy used to treat or prevent malnutrition. This includes any method of feeding intended to replace or support oral feeding.
    """
    id: str = Field(..., description="""A unique identifier for the named entity""")
    label: Optional[str] = Field(None, description="""The label (name) of the named thing""")
    
    

class Disease(NamedEntity):
    
    id: str = Field(..., description="""A unique identifier for the named entity""")
    label: Optional[str] = Field(None, description="""The label (name) of the named thing""")
    
    

class Drug(NamedEntity):
    
    id: str = Field(..., description="""A unique identifier for the named entity""")
    label: Optional[str] = Field(None, description="""The label (name) of the named thing""")
    
    

class Unit(NamedEntity):
    
    id: str = Field(..., description="""A unique identifier for the named entity""")
    label: Optional[str] = Field(None, description="""The label (name) of the named thing""")
    
    

class CompoundExpression(ConfiguredBaseModel):
    
    None
    
    

class QuantitativeValue(CompoundExpression):
    
    value: Optional[str] = Field(None, description="""The value of the quantity, or N/A if not provided.""")
    unit: Optional[str] = Field(None, description="""The unit of the quantity, or N/A if not provided.""")
    
    

class QuantitativeValueWithMetric(QuantitativeValue):
    
    percentile: Optional[str] = Field(None, description="""The reported percentile of the value, as compared to a reference patient population. Always positive, on a scale from 0 to 99%. May be reported as \"X%\", \"X%ile\", or \"Xth percentile\", where X is the value. N/A if not provided.""")
    zscore: Optional[str] = Field(None, description="""The relative standard deviation of the value, as a function of the percentile. May be positive or negative. May be reported as \"z-score\", \"Z-score\", or \"Z\", followed by the value. N/A if not provided.""")
    value: Optional[str] = Field(None, description="""The value of the quantity, or N/A if not provided.""")
    unit: Optional[str] = Field(None, description="""The unit of the quantity, or N/A if not provided.""")
    
    

class QuantitativeValueWithFrequency(QuantitativeValue):
    
    frequency: Optional[str] = Field(None, description="""A phrase describing how often an event or procedure should happen. May include time phrases such as \"per day\", Latin version of the same such as \"per diem\", single words such as \"daily\", or more complex phrases such as \"every 4 hours\". N/A if not provided.""")
    value: Optional[str] = Field(None, description="""The value of the quantity, or N/A if not provided.""")
    unit: Optional[str] = Field(None, description="""The unit of the quantity, or N/A if not provided.""")
    
    

class DietSupplementation(CompoundExpression):
    
    supplement: Optional[str] = Field(None, description="""The name of a specific material added to a patient's diet.""")
    amount: Optional[QuantitativeValueWithFrequency] = Field(None, description="""The quantity or dosage of the therapy, if provided. May include a frequency. N/A if not provided.""")
    dosage_by_unit: Optional[str] = Field(None, description="""The unit of a patient's properties used to determine supplement dosage. Often \"kilogram\". N/A if not provided.""")
    duration: Optional[QuantitativeValue] = Field(None, description="""The duration of the supplementation, if provided. N/A if not provided.""")
    route_of_administration: Optional[str] = Field(None, description="""The route of administration for the supplementation, if provided. N/A if not provided.""")
    
    

class NutritionSupport(CompoundExpression):
    
    method: Optional[str] = Field(None, description="""The name of a method used to provide nutritional support.""")
    components: Optional[List[NutritionSupportComponent]] = Field(default_factory=list, description="""The names of specific components included in a patient's diet.""")
    
    

class NutritionSupportComponent(CompoundExpression):
    
    material: Optional[str] = Field(None, description="""The name of a specific material included in a patient's diet.""")
    amount: Optional[QuantitativeValueWithFrequency] = Field(None, description="""The quantity or dosage of the therapy, if provided. May include a frequency. N/A if not provided.""")
    dosage_by_unit: Optional[str] = Field(None, description="""The unit of a patient's properties used to determine diet amounts. Often \"kilogram\". N/A if not provided.""")
    duration: Optional[QuantitativeValue] = Field(None, description="""The duration of the therapy, if provided. N/A if not provided.""")
    
    

class DrugTherapy(CompoundExpression):
    
    drug: Optional[str] = Field(None, description="""The name of a specific drug for a patient's preventative or therapeutic treatment.""")
    amount: Optional[QuantitativeValueWithFrequency] = Field(None, description="""The quantity or dosage of the drug, if provided. May include a frequency. N/A if not provided.""")
    dosage_by_unit: Optional[str] = Field(None, description="""The unit of a patient's properties used to determine drug dosage. Often \"kilogram\". N/A if not provided.""")
    duration: Optional[QuantitativeValue] = Field(None, description="""The duration of the drug therapy, if provided. N/A if not provided.""")
    route_of_administration: Optional[str] = Field(None, description="""The route of administration for the drug therapy, if provided. N/A if not provided.""")
    
    

class Triple(CompoundExpression):
    """
    Abstract parent for Relation Extraction tasks
    """
    subject: Optional[str] = Field(None)
    predicate: Optional[str] = Field(None)
    object: Optional[str] = Field(None)
    qualifier: Optional[str] = Field(None, description="""A qualifier for the statements, e.g. \"NOT\" for negation""")
    subject_qualifier: Optional[str] = Field(None, description="""An optional qualifier or modifier for the subject of the statement, e.g. \"high dose\" or \"intravenously administered\"""")
    object_qualifier: Optional[str] = Field(None, description="""An optional qualifier or modifier for the object of the statement, e.g. \"severe\" or \"with additional complications\"""")
    
    

class TextWithTriples(ConfiguredBaseModel):
    """
    A text containing one or more relations of the Triple type.
    """
    publication: Optional[Publication] = Field(None)
    triples: Optional[List[Triple]] = Field(default_factory=list)
    
    

class TextWithEntity(ConfiguredBaseModel):
    """
    A text containing one or more instances of a single type of entity.
    """
    publication: Optional[Publication] = Field(None)
    entities: Optional[List[str]] = Field(default_factory=list)
    
    

class RelationshipType(NamedEntity):
    
    id: str = Field(..., description="""A unique identifier for the named entity""")
    label: Optional[str] = Field(None, description="""The label (name) of the named thing""")
    
    

class Publication(ConfiguredBaseModel):
    
    id: Optional[str] = Field(None, description="""The publication identifier""")
    title: Optional[str] = Field(None, description="""The title of the publication""")
    abstract: Optional[str] = Field(None, description="""The abstract of the publication""")
    combined_text: Optional[str] = Field(None)
    full_text: Optional[str] = Field(None, description="""The full text of the publication""")
    
    

class AnnotatorResult(ConfiguredBaseModel):
    
    subject_text: Optional[str] = Field(None)
    object_id: Optional[str] = Field(None)
    object_text: Optional[str] = Field(None)
    
    


# Model rebuild
# see https://pydantic-docs.helpmanual.io/usage/models/#rebuilding-a-model
ClinicalObservationSet.model_rebuild()
MalnutritionObservations.model_rebuild()
ExtractionResult.model_rebuild()
NamedEntity.model_rebuild()
ClinicalObservations.model_rebuild()
TherapeuticMaterial.model_rebuild()
NutritionSupportMethod.model_rebuild()
Disease.model_rebuild()
Drug.model_rebuild()
Unit.model_rebuild()
CompoundExpression.model_rebuild()
QuantitativeValue.model_rebuild()
QuantitativeValueWithMetric.model_rebuild()
QuantitativeValueWithFrequency.model_rebuild()
DietSupplementation.model_rebuild()
NutritionSupport.model_rebuild()
NutritionSupportComponent.model_rebuild()
DrugTherapy.model_rebuild()
Triple.model_rebuild()
TextWithTriples.model_rebuild()
TextWithEntity.model_rebuild()
RelationshipType.model_rebuild()
Publication.model_rebuild()
AnnotatorResult.model_rebuild()




"""Clients for accessing different APIs."""

from ontogpt.clients.llm_client import LLMClient  # noqa:F401
from ontogpt.clients.pubmed_client import PubmedClient  # noqa:F401
from ontogpt.clients.soup_client import SoupClient  # noqa:F401
